/*
 * Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 * Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.ucsc.its.temerity.shared.ui.elements

import androidx.compose.desktop.ui.tooling.preview.Preview
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Edit
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.LinkAnnotation
import androidx.compose.ui.text.buildAnnotatedString
import androidx.compose.ui.text.withLink
import androidx.compose.ui.unit.dp
import com.composables.composetheme.ComposeTheme
import com.composables.composetheme.material3.colorScheme
import com.composables.composetheme.material3.typography
import edu.ucsc.its.temerity.remote.Course
import org.jetbrains.compose.resources.stringResource
import temerity.shared.compose.generated.resources.Res
import temerity.shared.compose.generated.resources.course_code_prefix
import temerity.shared.compose.generated.resources.course_edit
import temerity.shared.compose.generated.resources.direct_course_link_prefix
import temerity.shared.compose.generated.resources.term_code_prefix

@Preview
@Composable
fun CoursesListPreview() {
  val mockCourseList = listOf(
    Course(),
    Course(),
    Course(),
  )
  PreviewThemeWrapper {
    CoursesList(courseList = mockCourseList) {}
  }
}

@Composable
fun CoursesList(courseList: List<Course>, onEditClick: (Course) -> Unit) {
  ScrollableList(courseList) {
    CourseCardRow(course = it, onEditClick = { onEditClick(it) })
  }
}

@Composable
fun CourseCardRow(course: Course, onEditClick: () -> Unit) {
  CardRow(
    title = course.courseName,
    subtitle = stringResource(Res.string.course_code_prefix) + course.courseCode,
  ) {
    LazyColumn(
      verticalArrangement = Arrangement.spacedBy(16.dp),
      modifier = Modifier.height(190.dp),
      contentPadding = PaddingValues(vertical = 8.dp),
    ) {
      item {
        Row(
          verticalAlignment = Alignment.CenterVertically,
          horizontalArrangement = Arrangement.SpaceBetween,
          modifier = Modifier.fillMaxWidth(),
        ) {
          Column {
            Text(
              text = stringResource(Res.string.term_code_prefix) + course.courseTerm,
              color = ComposeTheme.colorScheme.primary,
              style = ComposeTheme.typography.bodyMedium,
            )
            Spacer(modifier = Modifier.width(4.dp).background(ComposeTheme.colorScheme.background))
            val directCourseLinkPrefix = stringResource(Res.string.direct_course_link_prefix)
            val courseLinkText = remember {
              buildAnnotatedString {
                append(directCourseLinkPrefix)
                withLink(
                  link = LinkAnnotation.Url(
                    url = course.directLink,
                  ),
                ) {
                  append(" ")
                  append(course.directLink)
                }
              }
            }
            Text(
              text = courseLinkText,
              color = ComposeTheme.colorScheme.primary,
              style = ComposeTheme.typography.bodyMedium,
            )
          }
          IconButton(
            onClick = onEditClick,
            modifier = Modifier.padding(8.dp),
          ) {
            Icon(imageVector = Icons.Default.Edit, contentDescription = stringResource(Res.string.course_edit))
          }
        }
      }
    }
  }
}
