/*
 * Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 * Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.ucsc.its.temerity.shared.di

import ca.gosyer.appdirs.AppDirs
import co.touchlab.kermit.Logger
import co.touchlab.kermit.koin.kermitLoggerModule
import edu.ucsc.its.temerity.platformClient
import edu.ucsc.its.temerity.shared.AppSettings
import edu.ucsc.its.temerity.shared.data.repository.PlatformRepository
import edu.ucsc.its.temerity.shared.data.storageModule
import edu.ucsc.its.temerity.shared.viewmodel.AppViewModel
import edu.ucsc.its.temerity.shared.viewmodel.LoginViewModel
import edu.ucsc.its.temerity.shared.viewmodel.RootViewModel
import edu.ucsc.its.temerity.shared.viewmodel.UserEditViewModel
import org.koin.compose.viewmodel.dsl.viewModelOf
import org.koin.core.context.startKoin
import org.koin.dsl.KoinAppDeclaration
import org.koin.dsl.koinApplication
import org.koin.dsl.module

fun initKoin(enableNetworkLogs: Boolean = false, appDeclaration: KoinAppDeclaration = {}) =
  startKoin {
    appDeclaration()
    modules(commonModule())
  }

internal fun commonModule(isDebuggingEnabled: Boolean = false) = module {
  viewModelOf(::AppViewModel)
  viewModelOf(::LoginViewModel)
  viewModelOf(::RootViewModel)
  viewModelOf(::UserEditViewModel)
  single { (serviceEndpoint: String, serviceToken: String) ->
    platformClient {
      debugEnabled(isDebuggingEnabled)
      serviceEndpoint(serviceEndpoint)
      serviceToken(serviceToken)
    }
  }
  single { PlatformRepository() }
  single { AppSettings(get()) }
  single<String> { (packageName: String) ->
    AppDirs("", packageName).getUserConfigDir()
  }
  includes(storageModule(), kermitLoggerModule(Logger.withTag("compose koin context")))
}

internal object AppKoinContext {
  private val koinApp = koinApplication {
    modules(commonModule())
  }

  val koin = koinApp.koin
}
