/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity

import co.touchlab.kermit.Logger
import com.skydoves.sandwich.StatusCode
import com.skydoves.sandwich.ktor.getStatusCode
import edu.ucsc.its.temerity.remote.NewUser
import edu.ucsc.its.temerity.remote.UserUpdate
import io.kotest.core.spec.style.FunSpec
import io.kotest.matchers.shouldBe
import io.ktor.http.HttpStatusCode
import kotlinx.coroutines.runBlocking
import org.dotenv.vault.dotenvVault

class DevUserApiTests : FunSpec({

  val dotenv = dotenvVault()

  lateinit var classUnderTest: PlatformClient

  beforeTest {
    classUnderTest = platformClient {
      debugEnabled(true)
      serviceEndpoint(dotenv["YUJADEV_API_URL"])
      serviceToken(dotenv["YUJADEV_TOKEN"])
    }
  }

  test("1.2.1 - PlatformClient can fetch a list of users") {
    runBlocking {
      val returnedUsers = classUnderTest.getUsers()
      println("Returned users:")
      returnedUsers.forEach { Logger.d(it.toString()) }
      assert(returnedUsers.isNotEmpty())
    }
  }

  test("1.2.2 - PlatformClient can fetch a single user") {
    runBlocking {
      val users = classUnderTest.getUsers()
      val userRef = users.find { user -> dotenv["YUJADEV_TEST_USER"] == user.loginId }
      requireNotNull(userRef)
      val user = classUnderTest.getUser(userRef.userId)
      Logger.d("Returned user with details:\n$user")
      assert(user.toString().isNotEmpty())
    }
  }

  test("1.2.3 - PlatformClient can create a user") {
    runBlocking {
      val newUserRef = NewUser(
        emailAddress = "wnwalker+3@ucsc.edu",
        firstName = "Billy",
        lastName = "Jones",
        loginId = "teststudent123244",
        loginPassword = "asdadfadf",
        userType = "Student",
      )
      val response = classUnderTest.createUser(newUser = newUserRef)
      Logger.d("response: $response")
      response.getStatusCode().shouldBe(StatusCode.OK)
    }
  }

  test("1.2.4. - PlatformClient can update user details") {
    val testUser = classUnderTest.getUsers().first { it.loginId == dotenv["YUJADEV_TEST_USER_ID"] }
    Logger.d("Previous platform value for test user last name: ${testUser.lastName}")
    runBlocking {
      val updateObject = UserUpdate()
      // Update the last name of the user by incrementing the current value
      val newLastName = testUser.lastName.toLong().inc().toString()
      updateObject.lastName = newLastName
      Logger.d("Attempting to update user last name with value: ${testUser.lastName}")
      val response = classUnderTest.updateUser(testUser.userId, updateObject)
      Logger.d("Received response: $response")
    }
  }

  test("1.2.5 - PlatformClient can delete a user") {
    runBlocking {
      val response = runBlocking { classUnderTest.deleteUser(dotenv["YUJADEV_TEST_USER_TO_DELETE"].toLong()) }
      Logger.d("response: $response")
      response.status.shouldBe(HttpStatusCode.OK)
    }
  }

  test("1.2.6 - PlatformClient can fetch a list of groups that a user is enrolled in") {
    runBlocking {
      val users = classUnderTest.getUsers()
      val userRef = users.find { user -> dotenv["YUJADEV_TEST_USER"] == user.loginId }
      requireNotNull(userRef)
      val userGroups = classUnderTest.getUserGroups(userRef.userId)
      Logger.d("Returned groups for the user ${userRef.loginId}:")
      userGroups.forEach { Logger.d(it.toString()) }
      assert(userGroups.isNotEmpty())
    }
  }

  test("1.2.7 - PlatformClient can fetch a list of groups that a user is content owner of") {
    runBlocking {
      val users = classUnderTest.getUsers()
      val userRef = users.find { user -> dotenv["YUJADEV_TEST_USER"] == user.loginId }
      requireNotNull(userRef)
      val userGroups = classUnderTest.getUserGroupsOwned(userRef.userId)
      Logger.d("Returned owned groups for the user ${userRef.loginId}:")
      userGroups.forEach { Logger.d(it.toString()) }
      assert(userGroups.isNotEmpty())
    }
  }

  test("1.2.8 - PlatformClient can create a new user with an external identifier set") {}

  test("1.2.9 - PlatformClient can add folder permissions") {}

  test("1.2.10 - PlatformClient can remove folder permissions") {}

  test("1.2.11 - PlatformClient can retrieve video content metadata") {}

  test("Extra - PlatformClient can retrieve a list of candidate test students to be deleted from the platform") {
    runBlocking {
      val users = classUnderTest.getUsers()
      val testStudents = users.filter { it.loginId.contains("tstudent") }
      testStudents.forEach {
        Logger.d("Test student found: $it\n")
      }
    }
  }
})
